@@ -4,7 +4,7 @@
 #' 
 #' @param x an \link[spatstat.explore]{fv.object}
 #' 
-#' @param key \link[base]{character} scalar, see function [keyval.fv()]
+#' @param key,.x \link[base]{character} scalars
 #' 
 #' @param ... additional parameters of functions 
 #' \link[stats]{loess}, except for `formula` and `data`
@@ -13,11 +13,16 @@
 #' @importFrom spatstat.explore fvnames as.data.frame.fv
 #' @importFrom stats loess
 #' @export
-loess.fv <- function(x, key = fvnames(fv, a = '.y'), ...) {
+loess.fv <- function(
+    x, 
+    key = fvnames(fv, a = '.y'), 
+    .x = fvnames(fv, a = '.x'),
+    ...
+) {
   fv.nm <- substitute(x)
   fv <- x; x <- NULL # make code more readable
-  .x <- fvnames(fv, a = '.x')
   force(key)
+  force(.x)
   if (key == .x) stop('first column of `x` is not the output of `fv.object`')
   fom <- eval(call(name = '~', as.symbol(key), as.symbol(.x)))
   lo <- loess(formula = fom, data = as.data.frame.fv(fv), ...)
